package io.syndesis.server.endpoint.v1.dto;

import com.google.common.base.MoreObjects;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import java.util.Optional;
import javax.annotation.CheckReturnValue;
import javax.annotation.Generated;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;

/**
 * Immutable implementation of {@link MetaData}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableMetaData.builder()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@Generated({"Immutables.generator", "MetaData"})
@Immutable
@CheckReturnValue
public final class ImmutableMetaData implements MetaData {
  private final @Nullable String message;
  private final @Nullable MetaData.Type type;

  private ImmutableMetaData(
      @Nullable String message,
      @Nullable MetaData.Type type) {
    this.message = message;
    this.type = type;
  }

  /**
   * @return The value of the {@code message} attribute
   */
  @Override
  public Optional<String> getMessage() {
    return Optional.ofNullable(message);
  }

  /**
   * @return The value of the {@code type} attribute
   */
  @Override
  public Optional<MetaData.Type> getType() {
    return Optional.ofNullable(type);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link MetaData#getMessage() message} attribute.
   * @param value The value for message
   * @return A modified copy of {@code this} object
   */
  public final ImmutableMetaData withMessage(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "message");
    if (Objects.equals(this.message, newValue)) return this;
    return new ImmutableMetaData(newValue, this.type);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link MetaData#getMessage() message} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for message
   * @return A modified copy of {@code this} object
   */
  public final ImmutableMetaData withMessage(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.message, value)) return this;
    return new ImmutableMetaData(value, this.type);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link MetaData#getType() type} attribute.
   * @param value The value for type
   * @return A modified copy of {@code this} object
   */
  public final ImmutableMetaData withType(MetaData.Type value) {
    @Nullable MetaData.Type newValue = Objects.requireNonNull(value, "type");
    if (this.type == newValue) return this;
    return new ImmutableMetaData(this.message, newValue);
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link MetaData#getType() type} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for type
   * @return A modified copy of {@code this} object
   */
  public final ImmutableMetaData withType(Optional<? extends MetaData.Type> optional) {
    @Nullable MetaData.Type value = optional.orElse(null);
    if (Objects.equals(this.type, value)) return this;
    return new ImmutableMetaData(this.message, value);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableMetaData} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableMetaData
        && equalTo((ImmutableMetaData) another);
  }

  private boolean equalTo(ImmutableMetaData another) {
    return Objects.equals(message, another.message)
        && Objects.equals(type, another.type);
  }

  /**
   * Computes a hash code from attributes: {@code message}, {@code type}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(message);
    h += (h << 5) + Objects.hashCode(type);
    return h;
  }

  /**
   * Prints the immutable value {@code MetaData} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return MoreObjects.toStringHelper("MetaData")
        .omitNullValues()
        .add("message", message)
        .add("type", type)
        .toString();
  }

  /**
   * Creates an immutable copy of a {@link MetaData} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable MetaData instance
   */
  public static ImmutableMetaData copyOf(MetaData instance) {
    if (instance instanceof ImmutableMetaData) {
      return (ImmutableMetaData) instance;
    }
    return ImmutableMetaData.builder()
        .from(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ImmutableMetaData ImmutableMetaData}.
   * @return A new ImmutableMetaData builder
   */
  public static ImmutableMetaData.Builder builder() {
    return new ImmutableMetaData.Builder();
  }

  /**
   * Builds instances of type {@link ImmutableMetaData ImmutableMetaData}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @NotThreadSafe
  public static final class Builder {
    private @Nullable String message;
    private @Nullable MetaData.Type type;

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code MetaData} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder from(MetaData instance) {
      Objects.requireNonNull(instance, "instance");
      Optional<String> messageOptional = instance.getMessage();
      if (messageOptional.isPresent()) {
        message(messageOptional);
      }
      Optional<MetaData.Type> typeOptional = instance.getType();
      if (typeOptional.isPresent()) {
        type(typeOptional);
      }
      return this;
    }

    /**
     * Initializes the optional value {@link MetaData#getMessage() message} to message.
     * @param message The value for message
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder message(String message) {
      this.message = Objects.requireNonNull(message, "message");
      return this;
    }

    /**
     * Initializes the optional value {@link MetaData#getMessage() message} to message.
     * @param message The value for message
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder message(Optional<String> message) {
      this.message = message.orElse(null);
      return this;
    }

    /**
     * Initializes the optional value {@link MetaData#getType() type} to type.
     * @param type The value for type
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder type(MetaData.Type type) {
      this.type = Objects.requireNonNull(type, "type");
      return this;
    }

    /**
     * Initializes the optional value {@link MetaData#getType() type} to type.
     * @param type The value for type
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder type(Optional<? extends MetaData.Type> type) {
      this.type = type.orElse(null);
      return this;
    }

    /**
     * Builds a new {@link ImmutableMetaData ImmutableMetaData}.
     * @return An immutable instance of MetaData
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableMetaData build() {
      return new ImmutableMetaData(message, type);
    }
  }
}
